from __future__ import annotations

from django.conf import settings
from django.templatetags.static import static
from django.utils.html import format_html


def django_htmx_script() -> str:
    # Optimization: whilst the script has no behaviour outside of debug mode,
    # don't include it.
    if not settings.DEBUG:
        return ""
    return format_html(
        '<script src="{}" data-debug="{}" defer></script>',
        static("django-htmx.js"),
        str(bool(settings.DEBUG)),
    )



from __future__ import annotations

import json
from typing import Any
from typing import Literal
from typing import TypeVar

from django.core.serializers.json import DjangoJSONEncoder
from django.http import HttpResponse
from django.http.response import HttpResponseBase
from django.http.response import HttpResponseRedirectBase

HTMX_STOP_POLLING = 286


class HttpResponseStopPolling(HttpResponse):
    status_code = HTMX_STOP_POLLING

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)
        self._reason_phrase = "Stop Polling"


class HttpResponseClientRedirect(HttpResponseRedirectBase):
    status_code = 200

    def __init__(self, redirect_to: str, *args: Any, **kwargs: Any) -> None:
        super().__init__(redirect_to, *args, **kwargs)
        self["HX-Redirect"] = self["Location"]
        del self["Location"]

    @property
    def url(self) -> str:
        return self["HX-Redirect"]


class HttpResponseClientRefresh(HttpResponse):
    def __init__(self) -> None:
        super().__init__()
        self["HX-Refresh"] = "true"


class HttpResponseLocation(HttpResponseRedirectBase):
    status_code = 200

    def __init__(
        self,
        redirect_to: str,
        *args: Any,
        source: str | None = None,
        event: str | None = None,
        target: str | None = None,
        swap: Literal[
            "innerHTML",
            "outerHTML",
            "beforebegin",
            "afterbegin",
            "beforeend",
            "afterend",
            "delete",
            "none",
            None,
        ] = None,
        select: str | None = None,
        values: dict[str, str] | None = None,
        headers: dict[str, str] | None = None,
        **kwargs: Any,
    ) -> None:
        super().__init__(redirect_to, *args, **kwargs)
        spec: dict[str, str | dict[str, str]] = {
            "path": self["Location"],
        }
        del self["Location"]
        if source is not None:
            spec["source"] = source
        if event is not None:
            spec["event"] = event
        if target is not None:
            spec["target"] = target
        if swap is not None:
            spec["swap"] = swap
        if select is not None:
            spec["select"] = select
        if headers is not None:
            spec["headers"] = headers
        if values is not None:
            spec["values"] = values
        self["HX-Location"] = json.dumps(spec)


_HttpResponse = TypeVar("_HttpResponse", bound=HttpResponseBase)


def push_url(response: _HttpResponse, url: str | Literal[False]) -> _HttpResponse:
    response["HX-Push-Url"] = "false" if url is False else url
    return response


def replace_url(response: _HttpResponse, url: str | Literal[False]) -> _HttpResponse:
    response["HX-Replace-Url"] = "false" if url is False else url
    return response


def reswap(response: _HttpResponse, method: str) -> _HttpResponse:
    response["HX-Reswap"] = method
    return response


def retarget(response: _HttpResponse, target: str) -> _HttpResponse:
    response["HX-Retarget"] = target
    return response


def trigger_client_event(
    response: _HttpResponse,
    name: str,
    params: dict[str, Any] | None = None,
    *,
    after: Literal["receive", "settle", "swap"] = "receive",
    encoder: type[json.JSONEncoder] = DjangoJSONEncoder,
) -> _HttpResponse:
    params = params or {}

    if after == "receive":
        header = "HX-Trigger"
    elif after == "settle":
        header = "HX-Trigger-After-Settle"
    elif after == "swap":
        header = "HX-Trigger-After-Swap"
    else:
        raise ValueError(
            "Value for 'after' must be one of: 'receive', 'settle', or 'swap'."
        )

    if header in response:
        value = response[header]
        try:
            data = json.loads(value)
        except json.JSONDecodeError as exc:
            raise ValueError(f"{header!r} value should be valid JSON.") from exc
        data[name] = params
    else:
        data = {name: params}

    response[header] = json.dumps(data, cls=encoder)

    return response



from __future__ import annotations

from typing import Any
from typing import cast

from django.core.handlers.wsgi import WSGIRequest
from django.http import HttpResponse
from django.http.response import HttpResponseBase
from django.test import RequestFactory as BaseRequestFactory
from django.test import SimpleTestCase

from django_htmx.middleware import HtmxDetails
from django_htmx.middleware import HtmxMiddleware


class HtmxWSGIRequest(WSGIRequest):
    htmx: HtmxDetails


class RequestFactory(BaseRequestFactory):
    def get(
        self, path: str, data: Any = None, secure: bool = False, **extra: Any
    ) -> HtmxWSGIRequest:
        return cast(HtmxWSGIRequest, super().get(path, data, secure, **extra))


def dummy_view(request):
    return HttpResponse("Hello!")


class HtmxMiddlewareTests(SimpleTestCase):
    request_factory = RequestFactory()
    middleware = HtmxMiddleware(dummy_view)

    def test_bool_default(self):
        request = self.request_factory.get("/")
        self.middleware(request)
        assert bool(request.htmx) is False

    def test_bool_false(self):
        request = self.request_factory.get("/", HTTP_HX_REQUEST="false")
        self.middleware(request)
        assert bool(request.htmx) is False

    def test_bool_true(self):
        request = self.request_factory.get("/", HTTP_HX_REQUEST="true")
        self.middleware(request)
        assert bool(request.htmx) is True

    def test_boosted_default(self):
        request = self.request_factory.get("/")
        self.middleware(request)
        assert not request.htmx.boosted

    def test_boosted_set(self):
        request = self.request_factory.get("/", HTTP_HX_BOOSTED="true")
        self.middleware(request)
        assert request.htmx.boosted

    def test_current_url_default(self):
        request = self.request_factory.get("/")
        self.middleware(request)
        assert request.htmx.current_url is None

    def test_current_url_set(self):
        request = self.request_factory.get(
            "/", HTTP_HX_CURRENT_URL="https://example.com"
        )
        self.middleware(request)
        assert request.htmx.current_url == "https://example.com"

    def test_current_url_set_url_encoded(self):
        request = self.request_factory.get(
            "/",
            HTTP_HX_CURRENT_URL="https%3A%2F%2Fexample.com%2F%3F",
            HTTP_HX_CURRENT_URL_URI_AUTOENCODED="true",
        )
        self.middleware(request)
        assert request.htmx.current_url == "https://example.com/?"

    def test_current_url_abs_path_default(self):
        request = self.request_factory.get("/")
        self.middleware(request)
        assert request.htmx.current_url_abs_path is None

    def test_current_url_abs_path_set_same_domain(self):
        request = self.request_factory.get(
            "/", HTTP_HX_CURRENT_URL="http://testserver/duck/?quack=true#h2"
        )
        self.middleware(request)
        assert request.htmx.current_url_abs_path == "/duck/?quack=true#h2"

    def test_current_url_abs_path_set_different_domain(self):
        request = self.request_factory.get(
            "/", HTTP_HX_CURRENT_URL="https://example.com/duck/?quack=true#h2"
        )
        self.middleware(request)
        assert request.htmx.current_url_abs_path is None

    def test_history_restore_request_false(self):
        request = self.request_factory.get("/", HTTP_HX_HISTORY_RESTORE_REQUEST="false")
        self.middleware(request)
        assert request.htmx.history_restore_request is False

    def test_history_restore_request_true(self):
        request = self.request_factory.get("/", HTTP_HX_HISTORY_RESTORE_REQUEST="true")
        self.middleware(request)
        assert request.htmx.history_restore_request is True

    def test_prompt_default(self):
        request = self.request_factory.get("/")
        self.middleware(request)
        assert request.htmx.prompt is None

    def test_prompt_set(self):
        request = self.request_factory.get("/", HTTP_HX_PROMPT="yes please")
        self.middleware(request)
        assert request.htmx.prompt == "yes please"

    def test_target_default(self):
        request = self.request_factory.get("/")
        self.middleware(request)
        assert request.htmx.target is None

    def test_target_set(self):
        request = self.request_factory.get("/", HTTP_HX_TARGET="some-element")
        self.middleware(request)
        assert request.htmx.target == "some-element"

    def test_trigger_default(self):
        request = self.request_factory.get("/")
        self.middleware(request)
        assert request.htmx.trigger is None

    def test_trigger_set(self):
        request = self.request_factory.get("/", HTTP_HX_TRIGGER="some-element")
        self.middleware(request)
        assert request.htmx.trigger == "some-element"

    def test_trigger_name_default(self):
        request = self.request_factory.get("/")
        self.middleware(request)
        assert request.htmx.trigger_name is None

    def test_trigger_name_set(self):
        request = self.request_factory.get("/", HTTP_HX_TRIGGER_NAME="some-name")
        self.middleware(request)
        assert request.htmx.trigger_name == "some-name"

    def test_triggering_event_none(self):
        request = self.request_factory.get("/")
        self.middleware(request)
        assert request.htmx.triggering_event is None

    def test_triggering_event_bad_json(self):
        request = self.request_factory.get("/", HTTP_TRIGGERING_EVENT="{")
        self.middleware(request)
        assert request.htmx.triggering_event is None

    def test_triggering_event_good_json(self):
        request = self.request_factory.get(
            "/",
            HTTP_TRIGGERING_EVENT="%7B%22target%22%3A%20null%7D",
            HTTP_TRIGGERING_EVENT_URI_AUTOENCODED="true",
        )
        self.middleware(request)
        assert request.htmx.triggering_event == {"target": None}

    async def test_async(self):
        async def dummy_async_view(request):
            return HttpResponse("Hello!")

        middleware = HtmxMiddleware(dummy_async_view)
        request = self.request_factory.get("/", HTTP_HX_REQUEST="true")

        result = middleware(request)
        assert not isinstance(result, HttpResponseBase)  # type narrow
        response = await result

        assert isinstance(response, HttpResponse)
        assert bool(request.htmx) is True



from __future__ import annotations

import json
from uuid import UUID

import pytest
from django.core.serializers.json import DjangoJSONEncoder
from django.http import HttpResponse
from django.http import StreamingHttpResponse
from django.test import SimpleTestCase

from django_htmx.http import HttpResponseClientRedirect
from django_htmx.http import HttpResponseClientRefresh
from django_htmx.http import HttpResponseLocation
from django_htmx.http import HttpResponseStopPolling
from django_htmx.http import push_url
from django_htmx.http import replace_url
from django_htmx.http import reswap
from django_htmx.http import retarget
from django_htmx.http import trigger_client_event


class HttpResponseStopPollingTests(SimpleTestCase):
    def test_success(self):
        response = HttpResponseStopPolling()

        assert response.status_code == 286
        assert response.reason_phrase == "Stop Polling"


class HttpResponseClientRedirectTests(SimpleTestCase):
    def test_success(self):
        response = HttpResponseClientRedirect("https://example.com")

        assert response.status_code == 200
        assert response["HX-Redirect"] == "https://example.com"
        assert "Location" not in response

    def test_repr(self):
        response = HttpResponseClientRedirect("https://example.com")

        assert repr(response) == (
            '<HttpResponseClientRedirect status_code=200, "text/html; '
            + 'charset=utf-8", url="https://example.com">'
        )


class HttpResponseLocationTests(SimpleTestCase):
    def test_success(self):
        response = HttpResponseLocation("/home/")

        assert response.status_code == 200
        assert "Location" not in response
        spec = json.loads(response["HX-Location"])
        assert spec == {"path": "/home/"}

    def test_success_complete(self):
        response = HttpResponseLocation(
            "/home/",
            source="#button",
            event="doubleclick",
            target="#main",
            swap="innerHTML",
            select="#content",
            headers={"year": "2022"},
            values={"banner": "true"},
        )

        assert response.status_code == 200
        assert "Location" not in response
        spec = json.loads(response["HX-Location"])
        assert spec == {
            "path": "/home/",
            "source": "#button",
            "event": "doubleclick",
            "target": "#main",
            "swap": "innerHTML",
            "select": "#content",
            "headers": {"year": "2022"},
            "values": {"banner": "true"},
        }


class HttpResponseClientRefreshTests(SimpleTestCase):
    def test_success(self):
        response = HttpResponseClientRefresh()

        assert response.status_code == 200
        assert response["Content-Type"] == "text/html; charset=utf-8"
        assert response["HX-Refresh"] == "true"


class PushUrlTests(SimpleTestCase):
    def test_success(self):
        response = HttpResponse()

        response2 = push_url(response, "/index.html")

        assert response2 is response
        assert response["HX-Push-Url"] == "/index.html"

    def test_success_false(self):
        response = HttpResponse()

        response2 = push_url(response, False)

        assert response2 is response
        assert response["HX-Push-Url"] == "false"


class ReplaceUrlTests(SimpleTestCase):
    def test_success(self):
        response = HttpResponse()

        response2 = replace_url(response, "/index.html")

        assert response2 is response
        assert response["HX-Replace-Url"] == "/index.html"

    def test_success_false(self):
        response = HttpResponse()

        response2 = replace_url(response, False)

        assert response2 is response
        assert response["HX-Replace-Url"] == "false"


class ReswapTests(SimpleTestCase):
    def test_success(self):
        response = HttpResponse()

        response2 = reswap(response, "outerHTML")

        assert response2 is response
        assert response["HX-Reswap"] == "outerHTML"


class RetargetTests(SimpleTestCase):
    def test_success(self):
        response = HttpResponse()

        response2 = retarget(response, "#heading")

        assert response2 is response
        assert response["HX-Retarget"] == "#heading"


class TriggerClientEventTests(SimpleTestCase):
    def test_fail_bad_after_value(self):
        response = HttpResponse()

        with pytest.raises(ValueError) as exinfo:
            trigger_client_event(
                response,
                "custom-event",
                {},
                after="bad-value",  # type: ignore [arg-type]
            )

        assert exinfo.value.args == (
            "Value for 'after' must be one of: 'receive', 'settle', or 'swap'.",
        )

    def test_fail_header_there_not_json(self):
        response = HttpResponse()
        response["HX-Trigger"] = "broken{"

        with pytest.raises(ValueError) as exinfo:
            trigger_client_event(response, "custom-event", {})

        assert exinfo.value.args == ("'HX-Trigger' value should be valid JSON.",)

    def test_success(self):
        response = HttpResponse()

        result = trigger_client_event(
            response, "showConfetti", {"colours": ["purple", "red"]}
        )

        assert result is response
        assert (
            response["HX-Trigger"] == '{"showConfetti": {"colours": ["purple", "red"]}}'
        )

    def test_success_no_params(self):
        response = HttpResponse()

        result = trigger_client_event(response, "showConfetti")

        assert result is response
        assert response["HX-Trigger"] == '{"showConfetti": {}}'

    def test_success_streaming(self):
        response = StreamingHttpResponse(iter((b"hello",)))

        result = trigger_client_event(
            response, "showConfetti", {"colours": ["purple", "red"]}
        )

        assert result is response
        assert (
            response["HX-Trigger"] == '{"showConfetti": {"colours": ["purple", "red"]}}'
        )

    def test_success_multiple_events(self):
        response = HttpResponse()

        result1 = trigger_client_event(
            response, "showConfetti", {"colours": ["purple"]}
        )
        result2 = trigger_client_event(response, "showMessage", {"value": "Well done!"})

        assert result1 is response
        assert result2 is response
        assert response["HX-Trigger"] == (
            '{"showConfetti": {"colours": ["purple"]},'
            + ' "showMessage": {"value": "Well done!"}}'
        )

    def test_success_override(self):
        response = HttpResponse()

        trigger_client_event(response, "showMessage", {"value": "That was okay."})
        trigger_client_event(response, "showMessage", {"value": "Well done!"})

        assert response["HX-Trigger"] == '{"showMessage": {"value": "Well done!"}}'

    def test_success_after_settle(self):
        response = HttpResponse()

        trigger_client_event(
            response, "showMessage", {"value": "Great!"}, after="settle"
        )

        assert (
            response["HX-Trigger-After-Settle"]
            == '{"showMessage": {"value": "Great!"}}'
        )

    def test_success_after_swap(self):
        response = HttpResponse()

        trigger_client_event(response, "showMessage", {"value": "Great!"}, after="swap")

        assert (
            response["HX-Trigger-After-Swap"] == '{"showMessage": {"value": "Great!"}}'
        )

    def test_django_json_encoder(self):
        response = HttpResponse()
        uuid_value = UUID("{12345678-1234-5678-1234-567812345678}")

        trigger_client_event(response, "showMessage", {"uuid": uuid_value})

        assert (
            response["HX-Trigger"]
            == '{"showMessage": {"uuid": "12345678-1234-5678-1234-567812345678"}}'
        )

    def test_custom_json_encoder(self):
        class Bean:
            pass

        class BeanEncoder(DjangoJSONEncoder):
            def default(self, o):
                if isinstance(o, Bean):
                    return "bean"

                return super().default(o)

        response = HttpResponse()

        trigger_client_event(
            response,
            "showMessage",
            {
                "a": UUID("{12345678-1234-5678-1234-567812345678}"),
                "b": Bean(),
            },
            encoder=BeanEncoder,
        )

        assert response["HX-Trigger"] == (
            '{"showMessage": {'
            + '"a": "12345678-1234-5678-1234-567812345678",'
            + ' "b": "bean"'
            + "}}"
        )



from __future__ import annotations

from django.test import SimpleTestCase
from django.test import override_settings

from django_htmx.jinja import django_htmx_script


class DjangoHtmxScriptTests(SimpleTestCase):
    def test_non_debug_empty(self):
        result = django_htmx_script()

        assert result == ""

    def test_debug_success(self):
        with override_settings(DEBUG=True):
            result = django_htmx_script()

        assert result == (
            '<script src="django-htmx.js" data-debug="True" defer></script>'
        )
